/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConfigurationDialog.h"

#include "Debug.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "OptionBrowsedLineEditor.h"
#include "OptionCheckBox.h"
#include "OptionSpinBox.h"
#include "TransparencyConfiguration.h"
#include "SvgConfiguration.h"
#include "SystemTrayIcon.h"

#include <QGroupBox>
#include <QLabel>
#include <QPushButton>

//_________________________________________________________
ConfigurationDialog::ConfigurationDialog( QWidget* parent, bool supportsSystemNotifications ):
BaseConfigurationDialog( parent )
{

    Debug::Throw( "ConfigurationDialog::ConfigurationDialog.\n" );
    setWindowTitle( "Configuration - Qssh" );
    setOptionName( "CONFIGURATION_DIALOG" );

    // base configuration
    baseConfiguration();

    // appearance
    {

        QWidget* page = &addPage( IconEngine::get( IconNames::PreferencesAppearance ), tr( "Appearance" ), tr( "Application Look and Feel" ) );

        // use system notifications
        OptionCheckBox* checkbox;
        page->layout()->addWidget( checkbox = new OptionCheckBox( tr( "Use system notifications" ), page, "USE_SYSTEM_NOTIFICATIONS" ) );
        addOptionWidget( checkbox );

        // message box
        QGroupBox* box = new QGroupBox( tr( "Message Box" ), page );
        page->layout()->addWidget( box );

        if( supportsSystemNotifications ) connect( checkbox, SIGNAL(toggled(bool)), box, SLOT(setDisabled(bool)) );
        else checkbox->hide();

        GridLayout* gridLayout = new GridLayout();
        gridLayout->setSpacing(5);
        gridLayout->setMargin(5);
        gridLayout->setMaxCount( 2 );
        gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
        box->setLayout( gridLayout );

        // margin
        OptionSpinBox* spinbox;
        QLabel* label;
        gridLayout->addWidget( label = new QLabel( tr( "Margin:" ), box ) );
        gridLayout->addWidget( spinbox = new OptionSpinBox( this, "WINDOW_BORDER" ) );
        spinbox->setSuffix( tr( "px" ) );
        spinbox->setMinimum( 0 );
        spinbox->setMaximum( 100 );
        spinbox->setToolTip( tr( "Distance between message box border and its contents" ) );
        addOptionWidget( spinbox );
        label->setBuddy( spinbox );

        // auto-hide
        gridLayout->addWidget( label = new QLabel( tr( "Auto-hide delay:" ), box ) );
        gridLayout->addWidget( spinbox = new OptionSpinBox( this, "MESSAGE_BOX_DELAY" ) );
        spinbox->setSuffix( tr( "s" ) );
        spinbox->setMinimum( 0 );
        spinbox->setMaximum( 100 );
        spinbox->setToolTip( tr( "Message box auto-hide delay. If 0, auto-hide is disabled" ) );
        addOptionWidget( spinbox );
        label->setBuddy( spinbox );

        // round corners
        gridLayout->setColumnAlignment( 0, Qt::AlignLeft|Qt::AlignVCenter );
        gridLayout->addWidget( checkbox = new OptionCheckBox( tr( "Round corners" ), box, "ROUND_CORNERS" ) );
        addOptionWidget( checkbox );
    }


    // transparency configuration
    QWidget* page;
    Transparency::TransparencyConfiguration* transparency_configuration;

    page = &addPage( IconEngine::get( IconNames::PreferencesColors ), tr( "Foreground" ), tr( "Foreground color settings" ) );
    page->layout()->addWidget( transparency_configuration = new Transparency::TransparencyConfiguration( page ));
    addOptionWidget( transparency_configuration );

    // svg background
    page = &addPage( IconEngine::get( IconNames::PreferencesBackground ), tr( "Background" ), tr( "Main window background settings" ) );
    Svg::SvgConfiguration* svgConfiguration( new Svg::SvgConfiguration( page ) );
    page->layout()->addWidget( svgConfiguration );
    addOptionWidget( svgConfiguration );

    // options
    {
        QWidget* page = &addPage( IconEngine::get( IconNames::PreferencesAppearance ), tr( "Behavior" ), tr( "Application's Behavior" ) );
        QWidget* box = new QWidget( page );
        page->layout()->addWidget( box );

        QVBoxLayout* vLayout = new QVBoxLayout();
        vLayout->setSpacing( 5 );
        vLayout->setMargin( 5 );
        box->setLayout( vLayout );

        OptionCheckBox* checkbox;
        vLayout->addWidget( checkbox = new OptionCheckBox( tr( "Automatically reconnect when network device changes" ), box, "NETWORK_AUTO_SELECT" ) );
        addOptionWidget( checkbox );

        QLabel* label;
        OptionSpinBox* spinbox;
        QHBoxLayout* hLayout = new QHBoxLayout();
        hLayout->setMargin(0);
        hLayout->setSpacing(5);
        vLayout->addLayout( hLayout );
        hLayout->addWidget( label = new QLabel( tr( "Network device check interval:" ), box ) );
        hLayout->addWidget( spinbox = new OptionSpinBox( box, "NETWORK_AUTO_SELECT_RATE" ) );
        spinbox->setSuffix( tr( "s" ) );
        spinbox->setMinimum( 1 );
        spinbox->setToolTip( tr( "Time interval (seconds) between checks" ) );
        addOptionWidget( spinbox );
        label->setBuddy( spinbox );

    }

    // load initial configuration
    read();

}
