/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"
#include "ConnectionParser.h"
#include "Debug.h"
#include "DefaultOptions.h"
#include "ResourceMigration.h"
#include "Singleton.h"
#include "SvgDefaultOptions.h"
#include "TransparencyDefaultOptions.h"
#include "XmlDef.h"
#include "XmlOptions.h"

#include <QApplication>

//__________________________________________
//! main function
int main (int argc, char *argv[])
{

    // install additional options
    installDefaultOptions();
    installSvgOptions();
    installTransparencyOptions();

    // migrate old rc files
    File oldRCFile( XmlOptions::get().raw( "OLD_RC_FILE" ) );
    File rcFile( XmlOptions::get().raw( "RC_FILE" ) );
    ResourceMigration( oldRCFile ).migrate( rcFile );

    // assign and read
    XmlOptions::setFile( rcFile );
    XmlOptions::read();

    // debug level
    Debug::setLevel( XmlOptions::get().get<int>( "DEBUG_LEVEL" ) );
    if( Debug::level() ) Debug::Throw() << XmlOptions::get() << endl;

    // resources
    Q_INIT_RESOURCE( basePixmaps );
    Q_INIT_RESOURCE( baseSvg );
    Q_INIT_RESOURCE( pixmaps );

    // application
    QApplication application( argc, argv );
    application.setQuitOnLastWindowClosed( false );

    // singleton application is deleted before QApplication
    const CommandLineArguments arguments( argc, argv );
    Application singleton( arguments );
    Singleton::get().setApplication( &singleton );

    const CommandLineParser parser( singleton.commandLineParser( arguments ) );
    if( parser.hasFlag( "--list" ) )
    {
        ConnectionParser parser( QString( XmlOptions::get().raw( "RC_FILE" ) ) );
        Ssh::ConnectionAttributes::List attributes = parser.read();
        if( parser.hasError() ) Debug::Throw(0) << parser.error() << endl;

        if( attributes.empty() )
        {
            Debug::Throw(0, QObject::tr( "No connections are presently registered.\n Righ-click to configure connections." ) );

        } else {

            for( const auto& connection:attributes )
            { Debug::Throw(0) << connection.name() << endl; }

        }

        return 0;

    }

    // initialize and run
    if( singleton.initApplicationManager() )
    {  application.exec(); }

    return 0;
}
